/*********************************************************************
下記のサイトに詳細情報を掲載しています。

http://www.geocities.jp/bokunimowakaru/diy/arduino/matrix_led.html

本ソースリストおよびソフトウェアは、ライセンスフリーです。
利用、編集、再配布等が自由に行えますが、著作権表示の改変は禁止します。

                               Copyright (c) 2014 Wataru KUNINO
                               http://www.geocities.jp/bokunimowakaru/

本スケッチの作成に下記の資料を参照しました。
http://playground.arduino.cc/Main/DirectDriveLEDMatrix
*********************************************************************/
#include <FrequencyTimer2.h>
#include <string.h>

#define	DISPLAY_COLS	240		// 表示用のバッファサイズ 最大256-16
#define	LED_COMMON		1		// LEDの極性。カソードがROW側(Aタイプ)は0、COL側(Bタイプ)は1
#define	RX_MAX			32		// シリアル受信バッファサイズ
#define	LED_DURATION	75		// LEDの点灯時間[ms]

// マトリクスLEDの1番ピンから16番ピンをArduinoのどのDigital入力ピンに接続するかを定義します。
// Arduinoの使いやすいピンをLEDに接続することが出来ます。14～19番ピンはAnalog 0～5に相当します。
// 下記のpins[17]の-1よりも後の数字を書き換えてください。

const static
//             /*LED    */  --  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16
byte pins[17]= /*Arduino*/ {-1,13,12,11,10,14,15,16,17, 2, 3, 4, 5, 6, 7, 8, 9};

/*
ブレッドボード用：
byte pins[17]=             {-1, 5, 4, 3, 2,14,15,16,17,13,12,11,10, 9, 8, 7, 6};

シールド製作用：
byte pins[17]=             {-1,13,12,11,10,14,15,16,17, 2, 3, 4, 5, 6, 7, 8, 9};
*/


// マトリクスLEDのピン番号と表示位置をLEDの仕様書から書き写してください。
// colsは列、rowsは行です。それぞれpins[番号]のように書きます。
const static
byte cols[8] = {pins[13],pins[ 3],pins[ 4],pins[10],pins[ 6],pins[11],pins[15],pins[16]};
const static
byte rows[8] = {pins[ 9],pins[14],pins[ 8],pins[12],pins[ 1],pins[ 7],pins[ 2],pins[ 5]};

////////////////////////////////////////////////////////////////////////////////////////////////

byte col = 0;
byte leds[DISPLAY_COLS];
byte pattern=0;
byte pattern_max=0;
char rx[RX_MAX+1];
byte rx_i=0;

void setup() {
	byte i;
	Serial.begin(9600);
	for(i=1;i<=16;i++) pinMode(pins[i], OUTPUT);			// sets the pins as output
	for(i=1;i<= 8;i++) digitalWrite(cols[i-1], LED_COMMON);	// set up cols and rows
	for(i=1;i<= 8;i++) digitalWrite(rows[i-1], LED_COMMON);
	clearLeds();
	FrequencyTimer2::disable();								// Turn off toggling of pin 11
	FrequencyTimer2::setPeriod(2000);						// Set refresh rate
	FrequencyTimer2::setOnOverflow(display);				// Set interrupt routine to be called
	sprintf(rx,"Hello! I'm Arduino.");
	setPattern(rx);
	Serial.println(rx);
}

void loop() {
	if (Serial.available() == 0) {							// シリアル入力に何も無い時
		pattern = ++pattern;								// LEDスクロール位置変更
		if(pattern > pattern_max){							// 最後まで表示したら
			if(rx_i>0){
				rx[rx_i]='\0';								// シリアルバッファに終了文字を入れる
				clearLeds();								// LED表示用のピクセルバッファを消去
				setPattern(rx);								// 受信した文字をセット
				Serial.println(rx);							// これから表示する内容をシリアルへ
				rx_i=0;										// 次の受信に備える
			}
			pattern=0;										// 最初に戻す
		}
		delay(LED_DURATION);								// スクロール用のウェイト
	}else{
		/*シリアルからの受信処理*/
		rx[rx_i]=Serial.read();								// 受信値をバッファへ保存
		if( isprint(rx[rx_i]) ){							// 表示可能な文字の時
			rx_i++;
			if(rx_i>RX_MAX) rx_i=RX_MAX;
		}else{												// 表示できない文字の時
			switch(rx[rx_i]){
				case '\n':									// 改行の時
					rx[rx_i]='\0';							// 終了文字を入れる
					/*
					while( ++pattern <= pattern_max )
						delay(LED_DURATION);				// 最後まで表示されるのを待つ
					setPattern(rx);							// 受信した文字をセット
					*/
					rx_i=RX_MAX;							// これ以上、受信しない
					break;
				default:
					break;
			}
		}
	
	}
}

void clearLeds() {
	for(int i = 0; i < DISPLAY_COLS; i++) leds[i] = 0x00;	// Clear display array
}

void setPattern(char *s) {
	int i=0,j;
	pattern=0;
	
	while(s[i] != '\0' && pattern < DISPLAY_COLS-8){
		pattern += Font_Draw(s[i],pattern,-1);
		i++;
	}
	pattern_max = pattern+16;
	pattern=0;
}

// Interrupt routine
void display() {
	byte row;
	digitalWrite(cols[col], LED_COMMON); 					// Turn whole previous column off
//	for(row = 0; row < 8; row++) digitalWrite(rows[row], LED_COMMON);
	col++;
	if (col == 8) col = 0;
	
	for (row = 0; row < 8; row++) {
		if(	col+pattern >= 8 &&
			col+pattern < pattern_max-8 &&
			(leds[col+pattern-8]>>row)&1 == 1	){
			digitalWrite(rows[row], LED_COMMON);			// Turn on this led
		} else digitalWrite(rows[row], !LED_COMMON);
	}
	digitalWrite(cols[col], !LED_COMMON);					// Turn whole column on at once

}
